@@ -1,5 +1,6 @@
 /*  MasqMail
     Copyright (C) 1999-2001 Oliver Kurth
+    Copyright (C) 2010 markus schnalke <meillo@marmaro.de>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -650,6 +651,8 @@ read_route(gchar * filename, gboolean is_local_net)
 			route->connect_error_fail = parse_boolean(rval);
 		else if (strcmp(lval, "do_correct_helo") == 0)
 			route->do_correct_helo = parse_boolean(rval);
+		else if (strcmp(lval, "instant_helo") == 0)
+			route->instant_helo = parse_boolean(rval);
 		else if (strcmp(lval, "do_pipelining") == 0)
 			route->do_pipelining = parse_boolean(rval);
 		else if (strcmp(lval, "allowed_return_paths") == 0)
